/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BSelect.java,v 1.18 2002/05/29 06:17:34 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.comp.model.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.apache.log4j.*;

/**
 * A BSelect element is used to render a list of items, and
 * to indicate which element(s) are selected. It is typically 
 * used to manipulate &lt;select&gt; elements.
 *
 * <p>In most cases you will not actually need to bind the component
 * to a view in order to use it--if you return it from a model, this
 * will be done for you automatically. If however, you intend to use
 * the component <em>standalone</em> (ie. manually attaching it to a 
 * specific node in the DOM) or <em>inline</em> (ie. in a toString()), 
 * then you MUST BIND IT TO A VIEW before rendering, or an error will 
 * be generated.
 */
public class BSelect extends BList {

    //public vars
    protected static Logger logger = Logger.getLogger(BSelect.class.getName());
    
    //private vars
    protected ListSelectionModel selectionModel = null;
    protected LocalModelListener scallback = null;
    protected List listeners = null;
    protected Integer viewSize = null;
    protected boolean disableBackButton = false;
    private ListenerFactory lnkListenerFactory;

    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BSelect() {}
    
    /**
     * Public constructor which creates the component and
     * associates it with a ListModel
     *
     * @param model the specific model to back this component
     */
    public BSelect(ListModel imodel) {
        this(imodel, null, null);
    }
    
    /**
     * Public constructor which creates the component and
     * binds it to a specific model. The component is also
     * bound to the specified view.
     *
     * <p>Null values may be passed in for any parameters, 
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually 
     * rendering the component
     *
     * @param model the specific model to back this component
     * @param view the View the component should be bound to
     * @param dvc the default ViewContext (opt--its presence allows the 
     *         component to be rendered as markup in toString())
     */
    BSelect(ListModel imodel, View iview, ViewContext idvc) {
        if (imodel!=null) setModel(imodel);
        if (iview!=null) this.addView(iview);    
        if (idvc!=null) setDefaultViewContext(idvc);
    }
    



    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BSelect.class, HTMLElement.class);
        installRendererFactory(rfHTML, BSelect.class, HTMLDocument.class);

    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLSelectRenderer();}
    }




    //--------------- BSelect -----------------------------
    /**
     * Set the selection model that backs the list. This causes 
     * the list to register as a listener on the model, so any 
     * changes to it will be reflected in the list. If no 
     * SelectionModel is specified, a DefaultSelectionModel will
     * be used.
     *
     * @param model the model that backs the list
     */
    public void setSelectionModel(ListSelectionModel imodel) {

        //deregister if possible
        if (selectionModel!=null && scallback!=null) {
            selectionModel.removeModelListener(scallback);
        }
    
        //set the model    
        selectionModel = imodel;
        
        //reregister if possible
        if (selectionModel!=null) {
            if (scallback==null) scallback = new LocalModelListener();
            selectionModel.addModelListener(scallback);
        }
        invalidate();
    }
    
    /**
     * Get the selection model that backs the list
     *
     * @return the selection model that backs the list
     */
    public ListSelectionModel getSelectionModel() {
        if (selectionModel==null) setSelectionModel(new DefaultListSelectionModel());
        return selectionModel;    
    }

    /**
     * Convenience method to get the selected index (if the
     * component allows multiple selections, this will return 
     * the index of the first selected item). Returns a -1 if 
     * there are no selected items
     *
     * @return the index of the first selected item
     */
    public int getSelectedIndex() {
        if (model==null || model.getSize()<1) return -1;
        if (selectionModel==null || selectionModel.isSelectionEmpty()) return -1;
        for (int i=0,max=model.getSize(); i<max; i++) {
            if (selectionModel.isSelectedIndex(i)) return i;
        }
        return -1;        
    }

    /**
     * Convenience method to set the selected index. Any other selected
     * indexes will be cleared first. Setting this value to -1
     * effectively clears all selections.
     *
     * @param i the index to be selected
     */
    public void setSelectedIndex(int i) {
        if (selectionModel==null) getSelectionModel();
        selectionModel.setSelectionInterval(i,i);
        invalidate();
    }

    /**
     * Convenience method to return an int array containing all
     * selected indexes. Returns a null if there are no selected items
     *
     * @return an int array for selected indexes
     */
    public int[] getSelectedIndexes() {
        if (model==null || model.getSize()<1) return null;
        if (selectionModel==null || selectionModel.isSelectionEmpty()) return null;
        int idx[] = new int[model.getSize()];
        int cntr = -1;
        for (int i=0,max=model.getSize(); i<max; i++) {
            if (selectionModel.isSelectedIndex(i)) idx[++cntr] = i;
        }
        int idx2[] = new int[cntr];
        System.arraycopy(idx, 0, idx2, 0, idx2.length); 
        return idx2;
    }

    /**
     * Set the view size of the component (heighth in rows). If this value
     * remains null, the height will default to whatever is specified in
     * the underlying markup. If you set this value, then the size will
     * be overridden in all the views associated with this component.
     *
     * Values less than 1 will be ignored.
     *
     * @param viewSize an integer value representing the size.
     */
    public void setViewSize(Integer iviewSize) {
        if (iviewSize==null || iviewSize.intValue()<1) return;
        viewSize = iviewSize;
        invalidate();
    }
    
    /**
     * Get the size of the component. May return a null if the size 
     * has not been manually specified. In this case, the renderer will
     * simply not set the size attribute in the underlying markup, leaving
     * it to default to whatever is already there.
     *
     * @return the view size
     */
    public Integer getViewSize() {
        return viewSize;
    }
    
    /**
     * Set disable back button (only works if your client supports
     * Javascript)
     *
     * @param disable true if we want the back button disabled 
     */
    public void setDisableBackButton(boolean idisableBackButton) {
        disableBackButton = idisableBackButton;
    }

    /**
     * Get disable back button
     *
     * @return true if we want the back button disabled 
     */
    public boolean getDisableBackButton() {
        return disableBackButton;
    }
    
    /**
     * Add an event listener to this component. 
     *
     * @param lf the event listener to be added
     */
    public void addEventListener(ListenerFactory lf) {
        if (lf==null) return;
        if (listeners==null) listeners = new ArrayList(5);
        listeners.add(lf);    
        invalidate();
    }
    
    /**
     * Remove an event listener from this component
     *
     * @param lf the event listener to be removed
     */
    public void removeEventListener(ListenerFactory lf) {
        if (lf==null) return;
        if (listeners==null) return;
        listeners.remove(lf);    
        invalidate();
    }

    /**
     * Here in the pre-render phase we actually add
     * BAction step children for any of the listeners
     * that might have been added to this component
     */
    protected void preRender(ViewContext vc, int depth) {
        //we want to actually add proxy components as step children
        //for any event listeners we might have to support
        if (listeners!=null) {
            //run through our list of listeners
            Iterator it = listeners.iterator();
            while (it.hasNext()) {
                ListenerFactory lf = (ListenerFactory) it.next();
            
                //run through our list of views
                Iterator it2 = getViews().iterator();
                while (it2.hasNext()) {
                    View v = (View) it2.next();
                    BAction baComp = new BAction();
                    baComp.addView(v);
                    baComp.addEventListener(lf);
                    this.addStepChild(baComp, true);
                }

                //csc_101001.1 - added
                //run through our list of temp views too (these
                //will be set if the component was returned from a 
                //model without being pre-bound to a view)
                if (tempViews!=null) {
                    it2 = tempViews.iterator();
                    while (it2.hasNext()) {
                        View v = (View) it2.next();
                        BAction baComp = new BAction();
                        baComp.addView(v);
                        baComp.addEventListener(lf);
                        this.addStepChild(baComp, true);
                    }
                }
            }
        }
    }

    /**
     * Render a specific view for the component. 
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
/*
//021102.3_csc - removed, because now its in BComponent     
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("rendering comp:"+this.toRef()+" view:"+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
            
        } catch (DOMException e) {
            logger.warn("DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }
*/




    //--------------- Lifecycle ----------------------------------
    /**
     * Destroy cycle. The component should use this cycle to
     * perform any special cleanup.
     */
    public void destroyCycle() {
        //default destroy
        super.destroyCycle();
    
        //we set the model to null so that the component can be 
        //garbage collected. If we don't do this, the model retains
        //a reference back to the component and so the component 
        //will never be freed up...
        setSelectionModel(null);        
    }




    //--------------- Utility ------------------------------------
/*
    class LocalListSelectionListener implements ListSelectionListener {
        //if the selection model changes, invalidate the component
        public void valueChanged(ListSelectionEvent e) {
            invalidate();
        }
    }
*/
    class LocalModelListener implements ModelListener {
        //get notified when one of the underlying models changes
        public void modelChanged(Model m) {
            invalidate();
        }
    }
    
}