/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLActionRenderer.java,v 1.28 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.scripting.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.http.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;

/**
 * This class handles the default rendering of actions into an HTML view.
 */
public class HTMLActionRenderer extends HTMLComponentRenderer {

    protected static Logger logger = Logger.getLogger(HTMLActionRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();
        
    /**
     * The purpose of this method is to create a default Node to be used when
     * the component is not bound to any specific view. In the case of BAction,
     * it will attempt to use the default template node if possible (ie. if
     * it is an <form>, <input>, <button>, or <select> element); if anything else, 
     * it will defer to the super() method
     *
     * @param doc the master Document which can be used to create elements
     *        from scratch
     * @param comp the component that we're dealing with for the current request
     * @param vc the view context for the current request
     * @return a default node (created from scratch)
     * @throws UnsupportedFormatException if the renderer has no default node
     */
//csc_110501.1    public Node createDefaultNode(Document doc, ViewContext vc) throws UnsupportedFormatException;
    public Node createDefaultNode(Document doc, BComponent comp, ViewContext vc) throws UnsupportedFormatException {  //csc_110501.1
        //ask the renderer to create the default Node
        Node templateNode = vc.getTemplateNode();
        Node defaultNode = null;
        if (templateNode instanceof HTMLAnchorElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLFormElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLInputElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLButtonElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else if (templateNode instanceof HTMLSelectElement) {
            defaultNode = templateNode.cloneNode(true);        
        } else {
            return super.createDefaultNode(doc, comp, vc);
        }
        if (logger.isInfoEnabled()) logger.info("Creating default node:"+defaultNode);
        return defaultNode;
    }

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a text component
        if (!(comp instanceof BAction)) throw new NoSuitableRendererException("This renderer can only render BAction components");
        BAction actionComp = (BAction) comp;
        Node node = view.getNode();
        ViewCapabilities vcap = vc.getViewCapabilities();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //..HTMLAnchorElement - set the "href" attribute
        if (node instanceof HTMLAnchorElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLAnchorElement interface...");
            
            //set the href
            ((HTMLAnchorElement) node).setHref(actionComp.getAction(vc));

            //if we need to disable the back button, and the client supports 
            //Javascript, we can do so by setting the onclick parameter
            if (actionComp.getDisableBackButton() && 
                vcap.getScriptingType() instanceof ScriptingType.JavaScript1x &&
                actionComp.getRenderStrategy()!=RenderStrategy.NEVER_SCRIPT &&  //csc_110201.1
                node instanceof Element) {
                    StringBuffer sb = new StringBuffer(200);
                    sb.append("location.replace(this.href); ");
                    sb.append("return false; ");
                    ((Element) node).setAttribute("onclick", sb.toString());                            
            }

            //make sure we prevent caching (we do this for NN in particular)
//csc_052002.1 - I am really not sure why we were doing this; if you go to a page with a form
//and submit it, and then try and go back, the form will always be expired (and this is not 
//necessarily what you want it to be all the time). So I'm going to remove this logic for now.
//If anyone remembers why we put this in here in the first place, please let me know...
//            actionComp.setPreventCaching(true);
            
        //..HTMLFormElement - set the "action" attribute
        } else if (node instanceof HTMLFormElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLFormElement interface...");
            HTMLFormElement fel = (HTMLFormElement) node;

            //if we need to disable the back button, and the client supports 
            //Javascript, we can do so by setting the action
            if (actionComp.getDisableBackButton() && 
                vcap.getScriptingType() instanceof ScriptingType.JavaScript1x &&
                actionComp.getRenderStrategy()!=RenderStrategy.NEVER_SCRIPT) {  //csc_110201.1

                //redirect the form to param gateway servlet
//csc_101701.1_start
//                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
//                fel.setAction(ParamGateway.PARAM_TARGET+lu.getBaseStr()+ParamGateway.PARAM_EXT+lu.getParamStr());
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc, true));
                String actionUrl = URLRewriter.encodeURL(vc, ParamGateway.PARAM_TARGET+lu.getBaseStr()+ParamGateway.PARAM_EXT)+lu.getParamStr();
                fel.setAction(actionUrl);
//csc_101701.1_end
                
                //set the onsubmit attribute
                StringBuffer sb = new StringBuffer(200);
                sb.append("cshl_SubmitAndReplace(document."+NamingHelper.getName(fel)+"); "); //csc_100201.2
                sb.append("return false; ");
                fel.setAttribute("onsubmit",sb.toString());
                
                //make sure we include the necessary script library
                String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/ClientServerHTTPLib.js";
                BScriptResource bsr = new BScriptResource(scr);
                bsr.setView(new DefaultView(node));
                actionComp.addStepChild(bsr, true);
                
                //make sure we prevent caching (we do this for NN in particular)
//csc_052002.1 - I am really not sure why we were doing this; if you go to a page with a form
//and submit it, and then try and go back, the form will always be expired (and this is not 
//necessarily what you want it to be all the time). So I'm going to remove this logic for now.
//If anyone remembers why we put this in here in the first place, please let me know...
//                actionComp.setPreventCaching(true);

            //else just set the form action
            } else {
                fel.setAction(actionComp.getAction(vc));
            }

        //..HTMLInputElement - 
        } else if (node instanceof HTMLInputElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");
            HTMLInputElement iel = (HTMLInputElement) node;
            HTMLFormElement fel = iel.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)

            //if the client supports Javascript...
//            if (false) {
            if (vcap.getScriptingType() instanceof ScriptingType.JavaScript1x &&
                actionComp.getRenderStrategy()!=RenderStrategy.NEVER_SCRIPT) {  //csc_110201.1
                
                //build the action String
                StringBuffer sb = new StringBuffer(200);
                StringBuffer sbAction = new StringBuffer(100);
                boolean disable = actionComp.getDisableBackButton();

                //csc_100201.1
                //make sure we disable the input element, thus preventing duplicate
                //submits by double clicking (note that this won't work in NN4)
                sb.append("this.disabled=true; ");
            
                //adjust the documents action 
//csc_101701.1_start
//                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
//                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
//                sbAction.append(lu.getBaseStr());
//                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
//                sbAction.append(lu.getParamStr());
//                sb.append("document."+NamingHelper.getName(fel)+".action='"+sbAction.toString()+"';");  //csc_100201.2
                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc, true));
                sbAction.append(lu.getBaseStr());
                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                String actionUrl = sbAction.toString();
                sb.append("document."+NamingHelper.getName(fel)+".action='"+URLRewriter.encodeURL(vc, actionUrl)+lu.getParamStr()+"';");
//csc_101701.1_end

                //adjust the onclick value to invoke submit
                if (disable) sb.append("cshl_SubmitAndReplace(document."+NamingHelper.getName(fel)+"); return false;");  //csc_100201.2
                else sb.append("document."+NamingHelper.getName(fel)+".submit(); return false;");  //csc_100201.2
                iel.setAttribute("onclick", sb.toString());
                
                //csc_101201.1 - added
                //if we are dealing with a "submit" button, we need to set the 
                //forms' action as well, since if the user hits enter, the form 
                //will be submitted but the button will not be clicked (IE actually
                //seems to handle this correctly, but Mozilla does not; this will
                //work for both). Thanks to Iman L. Crawford [icrawford@greatnation.com]
                //for pointing out this problem.
                if (iel.getType().equals("submit")) {
                    String fcmd = "this."+NamingHelper.getName(iel)+".click(); return false;";
                    fel.setAttribute("onsubmit", fcmd);
                }
            
                //make sure we include the necessary script library
                if (disable) {
                    String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/ClientServerHTTPLib.js";
                    BScriptResource bsr = new BScriptResource(scr);
                    bsr.setView(new DefaultView(node));
                    actionComp.addStepChild(bsr, true);
                }
                
                //make sure we prevent caching (we do this for NN in particular)
//csc_052002.1 - I am really not sure why we were doing this; if you go to a page with a form
//and submit it, and then try and go back, the form will always be expired (and this is not 
//necessarily what you want it to be all the time). So I'm going to remove this logic for now.
//If anyone remembers why we put this in here in the first place, please let me know...
//                actionComp.setPreventCaching(true);
                                
            //else do it the manual way    
            } else {
                //make sure the button type is submit
                if (!iel.getType().toLowerCase().equals("submit")) {
                    String errmsg = "Cannot render Input action listener; input type is not 'submit': "+node;
                    logger.warn(errmsg);
                    throw new NoSuitableRendererException(errmsg);
                }
            
//csc_101701.1_start
                //rename the button appropriately (don't encode)
//                iel.setName(FormGateway.FORM_TARGET+actionComp.getAction(vc));            
                iel.setName(FormGateway.FORM_TARGET+actionComp.getAction(vc, true));
                
                //redirect the form to event forwarding servlet (encode)
//                fel.setAction(NamingHelper.getName(fel)+FormGateway.FORM_EXT);  //csc_100201.2
                fel.setAction(URLRewriter.encodeURL(vc, NamingHelper.getName(fel)+FormGateway.FORM_EXT));
//csc_101701.1_end
            }

        //..HTMLButtonElement - 
        } else if (node instanceof HTMLButtonElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLButtonElement interface...");
            HTMLButtonElement bel = (HTMLButtonElement) node;
            HTMLFormElement fel = bel.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)
            
            //if the client supports Javascript...
            if (vcap.getScriptingType() instanceof ScriptingType.JavaScript1x &&
                actionComp.getRenderStrategy()!=RenderStrategy.NEVER_SCRIPT) {  //csc_110201.1
                
                //build the action String
                StringBuffer sb = new StringBuffer(200);
                StringBuffer sbAction = new StringBuffer(100);
                boolean disable = actionComp.getDisableBackButton();

                //csc_100201.1
                //make sure we disable the input element, thus preventing duplicate
                //submits by double clicking (note that this won't work in NN4)
                sb.append("this.disabled=true; ");
            
                //adjust the documents action 
//csc_101701.1_start
//                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
//                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
//                sbAction.append(lu.getBaseStr());
//                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
//                sbAction.append(lu.getParamStr());
//                sb.append("document."+NamingHelper.getName(fel)+".action='"+sbAction.toString()+"';");  //csc_100201.2
                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc, true));
                sbAction.append(lu.getBaseStr());
                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                String actionUrl = sbAction.toString();
                sb.append("document."+NamingHelper.getName(fel)+".action='"+URLRewriter.encodeURL(vc, actionUrl)+lu.getParamStr()+"';");
//csc_101701.1_end
            
                //adjust the onclick value to invoke submit
                if (disable) sb.append("cshl_SubmitAndReplace(document."+NamingHelper.getName(fel)+"); return false;");  //csc_100201.2
                else sb.append("document."+NamingHelper.getName(fel)+".submit(); return false;");  //csc_100201.2
                bel.setAttribute("onclick", sb.toString());
            
                //make sure we include the necessary script library
                if (disable) {
                    String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/ClientServerHTTPLib.js";
                    BScriptResource bsr = new BScriptResource(scr);
                    bsr.setView(new DefaultView(node));
                    actionComp.addStepChild(bsr, true);
                }
                
                //make sure we prevent caching (we do this for NN in particular)
//csc_052002.1 - I am really not sure why we were doing this; if you go to a page with a form
//and submit it, and then try and go back, the form will always be expired (and this is not 
//necessarily what you want it to be all the time). So I'm going to remove this logic for now.
//If anyone remembers why we put this in here in the first place, please let me know...
//                actionComp.setPreventCaching(true);

            //else do it the manual way    
            } else {
                //make sure the button type is submit
                if (!bel.getType().toLowerCase().equals("submit")) {
                    String errmsg = "Cannot render Button action listener; button type is not 'submit': "+node;
                    logger.warn(errmsg);
                    throw new NoSuitableRendererException(errmsg);
                }
            
//csc_101701.1_start
                //rename the button appropriately (don't encode)
//                bel.setName(FormGateway.FORM_TARGET+actionComp.getAction(vc));            
                bel.setName(FormGateway.FORM_TARGET+actionComp.getAction(vc, true));
                
                //redirect the form to event forwarding servlet (encode)
//                fel.setAction(NamingHelper.getName(fel)+FormGateway.FORM_EXT);  //csc_100201.2
                fel.setAction(URLRewriter.encodeURL(vc, NamingHelper.getName(fel)+FormGateway.FORM_EXT));
//csc_101701.1_end
            }
            
        //..HTMLSelectElement - 
        } else if (node instanceof HTMLSelectElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLSelectElement interface...");
            HTMLSelectElement sel = (HTMLSelectElement) node;
            HTMLFormElement fel = sel.getForm();
//TODO:need to handle elements that are not part of a form (ie. fel==null)
            
            //if the client supports Javascript...
            if (vcap.getScriptingType() instanceof ScriptingType.JavaScript1x &&
                actionComp.getRenderStrategy()!=RenderStrategy.NEVER_SCRIPT) {  //csc_110201.1
                
                //build the action String
                StringBuffer sb = new StringBuffer(200);
                StringBuffer sbAction = new StringBuffer(100);
                boolean disable = actionComp.getDisableBackButton();

                //adjust the documents action 
//csc_101701.1_start
//                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
//                LightweightURL lu = new LightweightURL(actionComp.getAction(vc));
//                sbAction.append(lu.getBaseStr());
//                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
//                sbAction.append(lu.getParamStr());
//                sb.append("document."+NamingHelper.getName(fel)+".action='"+sbAction.toString()+"';");  //csc_100201.2
                if (disable) sbAction.append(ParamGateway.PARAM_TARGET);
                LightweightURL lu = new LightweightURL(actionComp.getAction(vc, true));
                sbAction.append(lu.getBaseStr());
                if (disable) sbAction.append(ParamGateway.PARAM_EXT);
                String actionUrl = sbAction.toString();
                sb.append("document."+NamingHelper.getName(fel)+".action='"+URLRewriter.encodeURL(vc, actionUrl)+lu.getParamStr()+"';");
//csc_101701.1_end
            
                //adjust the onclick value to invoke submit
                if (disable) sb.append("cshl_SubmitAndReplace(document."+NamingHelper.getName(fel)+"); return false;");  //csc_100201.2
                else sb.append("document."+NamingHelper.getName(fel)+".submit(); return false;");  //csc_100201.2
                sel.setAttribute("onchange", sb.toString());
            
                //make sure we include the necessary script library
                if (disable) {
                    String scr = ResourceGateway.EXT_RESOURCE_ID+"/org/enhydra/barracuda/core/scripts/ClientServerHTTPLib.js";
                    BScriptResource bsr = new BScriptResource(scr);
                    bsr.setView(new DefaultView(node));
                    actionComp.addStepChild(bsr, true);
                }
                
                //make sure we prevent caching (we do this for NN in particular)
//csc_052002.1 - I am really not sure why we were doing this; if you go to a page with a form
//and submit it, and then try and go back, the form will always be expired (and this is not 
//necessarily what you want it to be all the time). So I'm going to remove this logic for now.
//If anyone remembers why we put this in here in the first place, please let me know...
//                actionComp.setPreventCaching(true);
                
            //else throw an error
            } else {
                String errmsg = "Cannot render Select action listener; client does not support Javascript: "+node;
                logger.warn(errmsg);
                throw new NoSuitableRendererException(errmsg);
            }

        } else {
            String errmsg = "Node does not implement HTMLAnchorElement, HTMLFormElement, HTMLInputElement or HTMLButtonElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
        
        //finally, make sure we reflect the components enabled/disabled status
        eh.setEnabled(node, comp.isEnabled());
    }
    
}