/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BScript.java,v 1.3 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.comp.scripting;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;


//csc_102201.1 - created
/**
 * BScript is used to attach a javascript command to a DOM element
 * attribute.
 *
 * <p>In most cases you will not actually need to bind the component
 * to a view in order to use it--if you return it from a model, this
 * will be done for you automatically. If however, you intend to use
 * the component <em>standalone</em> (ie. manually attaching it to a
 * specific node in the DOM) or <em>inline</em> (ie. in a toString()),
 * then you MUST BIND IT TO A VIEW before rendering, or an error will
 * be generated.
 */
public class BScript extends BComponent {

    //public vars
    protected static Logger logger = Logger.getLogger(BScript.class.getName());

    //how should the command be inserted
    public static final int REPLACE = 0;
    public static final int PREPEND = 1;
    public static final int APPEND = 2;

    //target actions
    //...std events
    public static final String ON_CLICK = "onclick";
    public static final String ON_DBL_CLICK = "ondblclick";
    public static final String ON_MOUSE_DOWN = "onmousedown";
    public static final String ON_MOUSE_UP = "onmouseup";
    public static final String ON_MOUSE_OVER = "onmouseover";
    public static final String ON_MOUSE_MOVE = "onmousemove";
    public static final String ON_MOUSE_OUT = "onmouseout";
    public static final String ON_KEY_PRESS = "onkeypress";
    public static final String ON_KEY_DOWN = "onkeydown";
    public static final String ON_KEY_UP = "onkeyup";
    //..<body> specific
    public static final String ON_LOAD = "onload";
    public static final String ON_UNLOAD = "onunload";
    //..<a>,<area>,<label>,<input>,<select>,<textarea>,<button> specific
    public static final String ON_FOCUS = "onfocus";
    public static final String ON_BLUR = "onblur";
    //..<input>,<textarea> specific
    public static final String ON_SELECT = "onselect";
    //..<input>,<select> specific
    public static final String ON_CHANGE = "onchange";
    //..<form> specific
    public static final String ON_SUBMIT = "onsubmit";
    public static final String ON_RESET = "onreset";

    //private vars
    protected String jscmd = null;
    protected String jsattr = null;
    protected int mode = REPLACE;
    protected List resources = null;

    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BScript() {}

    /**
     * Public constructor which creates the component. When rendered,
     * it will take the specified command and REPLACE it in the target
     * scripting attribute
     *
     * <p>You should generally only use this constructor when returning
     * BScript from a Model, as the model components will automatically
     * bind the component to a view for you. If you use this constructor
     * in some other situation, you should manually bind the component
     * to the proper view.
     *
     * @param jsattr the target script attribute
     * @param jscmd the script command
     */
    public BScript(String jsattr, String jscmd) {
        this(jsattr, jscmd, REPLACE);
    }

    /**
     * Public constructor which creates the component. When rendered,
     * it will take the specified command and render it in the target
     * scripting attribute, using the specified replacement mode.
     *
     * @param jsattr the target script attribute
     * @param jscmd the script command
     * @param mode the cmd replacement mode (REPLACE, PREPEND, or APPEND)
     */
    public BScript(String jsattr, String jscmd, int mode) {
        this (jsattr, jscmd, mode, null);
    }

    /**
     * Public constructor which creates the component. When rendered,
     * it will take the specified command and render it in the target
     * scripting attribute, using the specified replacement mode.
     *
     * @param jsattr the target script attribute
     * @param jscmd the script command
     * @param mode the cmd replacement mode (REPLACE, PREPEND, or APPEND)
     * @param resource any necessary resource scripts
     */
    public BScript(String jsattr, String jscmd, int mode, String resource) {
        if (jsattr!=null) setAttr(jsattr);
        if (jscmd!=null) setCmd(jscmd);
        setMode(mode);
        if (resource!=null) addResource(resource);
    }

    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BScript.class, HTMLElement.class);
        installRendererFactory(rfHTML, BScript.class, HTMLDocument.class);
/*
        WMLRendererFactory rfWML = new WMLRendererFactory();
        installRendererFactory(rfWML, BScript.class, HTMLElement.class);
        installRendererFactory(rfWML, BScript.class, HTMLDocument.class);
*/
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLScriptRenderer();}
    }



    //--------------- BComponent ---------------------------------
    /**
     * Specify the JavaScript attr the command should be bound to
     *
     * @param jsattr the JavaScript attr the command should be bound to
     */
    public void setAttr(String ijsattr) {
        jsattr = ijsattr;
        invalidate();
    }

    /**
     * Get the JavaScript attr the command is be bound to
     *
     * @return the JavaScript attr the command is be bound to
     */
    public String getAttr() {
        return jsattr;
    }

    /**
     * Set the JavaScript command for this particular component
     *
     * @param jscmd the JavaScript command that backs this component
     */
    public void setCmd(String ijscmd) {
        jscmd = ijscmd;
        invalidate();
    }

    /**
     * Get the JavaScript command for this particular component
     *
     * @return the JavaScript command for this particular component
     */
    public String getCmd() {
        return jscmd;
    }

    /**
     * Set the mode for the action (defaults to REPLACE). In most cases,
     * you will simply REPLACE whatever the current cmd is in the template.
     * Sometimes, however, you may want to keep what's already there. In such
     * a situation, you would specify either APPEND (tack it on to what's
     * already there) or PREPEND (stick it in front of what's there)
     *
     * @param mode the cmd replacement mode
     */
    public void setMode(int imode) {
        if (imode!=PREPEND && imode!=APPEND) imode = REPLACE;
        mode = imode;
        invalidate();
    }

    /**
     * Get the replacement mode for this script
     *
     * @return the replacement mode for this script
     */
    public int getMode() {
        return mode;
    }

    /**
     * Add a script resource (resources specified will automatically
     * create BScriptResource delegates at render time)
     *
     * @param a script resource
     */
    public void addResource(String resource) {
        if (resources==null) resources = new ArrayList();
        resources.add(resource);
        invalidate();
    }

    /**
     * Remove a script resource
     *
     * @param a script resource
     */
    public void removeResource(String resource) {
        if (resources==null) return;
        resources.remove(resource);
        invalidate();
    }

    /**
     * Get a list of script resource
     *
     * @return a list of script resources
     */
    public List getResources() {
        return resources;
    }


    /**
     * Render a specific view for the component.
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
/*
//021102.3_csc - removed, because now its in BComponent     
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("rendering view: "+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
        } catch (DOMException e) {
            logger.warn("DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }
*/
    /**
     * Get a String representation of the component
     */
    public String toString() {
        return jsattr+"=\""+jscmd+"\"";
    }

}