/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultEventContext.java,v 1.14 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.view.*;
import org.apache.log4j.*;

/**
 * <p>The context contains information about the event (event, 
 * queue, plus request and response info if appropriate). The
 * context also implements StateMap, so you can use it to pass 
 * information between event handlers. The scope of the context 
 * is only for the duration of an event dispatch cycle (ie. from 
 * Http Request to Http Response)
 *
 * <p>Note that because DefaultEventContext implements ViewEventContext
 * it can safely be cast to its two parent interfaces: EventContext and
 * ControlEventContext. This is in fact what the DefaultEventDispatcher
 * actually does to control access to the appropriate information 
 * (ie. a ControlEvent handler should get a reference to an ControlEventContext,
 * NOT an ViewEventContext, since the control handlers shouldn't have
 * access to the HttpServletResponse object.)
 */
public class DefaultEventContext implements ViewEventContext {

    //public vars
    protected static Logger logger = Logger.getLogger(DefaultEventContext.class.getName());


    //private vars
    private static final String STATE = DefaultEventContext.class.getName()+".Statemap";
    private static final String LOCAL_OR = DefaultEventContext.class.getName()+".LocalOR";  //csc_022502.1
    private StateMap statemap = null;

    /**
     * Public constructor. 
     *
     * @param queue the event dispatch queue
     * @param config the servlet config structure
     * @param req the HttpServletRequest
     * @param resp the HttpServlet Response
     * @param defaultResponseEvent the default response event to be used 
     *        if none is generated by the control event handlers
     */    
    public DefaultEventContext(DispatchQueue queue, ViewCapabilities vc, ServletConfig config, HttpServletRequest req, HttpServletResponse resp, HttpResponseEvent defaultResponseEvent) {
        restoreContext(req);
        this.putState(EventContext.DISPATCH_QUEUE, queue);    
        this.putState(EventContext.VIEW_CAPABILITIES, vc);    
        this.putState(ControlEventContext.SERVLET_CONFIG, config);    
        this.putState(ControlEventContext.HTTP_SERVLET_REQUEST, req);    
        this.putState(ViewEventContext.HTTP_SERVLET_RESPONSE, resp);    
        this.putState(DefaultEventDispatcher.DEFAULT_RESPONSE_EVENT, defaultResponseEvent);
    }

    /**
     * Get the underlying BaseEvent
     *
     * @return the underlying BaseEvent
     */
    public BaseEvent getEvent() {
        return (BaseEvent) this.getState(EventContext.BASE_EVENT);    
    }

    /**
     * Get the underlying DispatchQueue
     *
     * @return the underlying DispatchQueue
     */
    public DispatchQueue getQueue() {
        return (DispatchQueue) this.getState(EventContext.DISPATCH_QUEUE);    
    }
    
    /**
     * Get the underlying ViewCapabilities
     */
    public ViewCapabilities getViewCapabilities() {
        return (ViewCapabilities) this.getState(EventContext.VIEW_CAPABILITIES);
    }
    
    /**
     * Get the associated ServletConfig structure
     *
     * @return the associated ServletConfig structure
     */
    public ServletConfig getConfig() {
        return (ServletConfig) this.getState(ControlEventContext.SERVLET_CONFIG);    
    }
    
    /**
     * Get the associated HttpServletRequest
     *
     * @return the associated HttpServletRequest
     */
    public HttpServletRequest getRequest() {
        return (HttpServletRequest) this.getState(ControlEventContext.HTTP_SERVLET_REQUEST);    
    }

    /**
     * Get the associated HttpServletResponse
     *
     * @return the associated HttpServletResponse
     */
    public HttpServletResponse getResponse() {
        return (HttpServletResponse) this.getState(ViewEventContext.HTTP_SERVLET_RESPONSE);    
    }
    
    /**
     * The event context must be able to persist its state 
     * in the user's session
     */
    public void persistContext() {
        SessionServices.getSession(getRequest()).setAttribute(STATE, statemap);
        //csc_022502.1_start
        ObjectRepository lr = ObjectRepository.getLocalRepository();
        List list = lr.getStateKeys();
        if (list!=null && list.size()>0) {
            StateMap sm = new DefaultStateMap();
            SessionServices.getSession(getRequest()).setAttribute(LOCAL_OR, sm);
            Iterator it = list.iterator();
            while (it.hasNext()) {
                Object key = it.next();
                Object val = lr.getState(key);
                sm.putState(key, val);
            }
        }
        //csc_022502.1_end
    }

    /**
     * The event context must also be able to restore state
     * from the user's session. Calling this method effectively
     * removes the state from the session. 
     */
    public void restoreContext(HttpServletRequest req) {
        HttpSession session = SessionServices.getSession(req, false);
        if (session!=null) {
            statemap = (StateMap) session.getAttribute(STATE);    
            session.removeAttribute(STATE);
        }
        if (statemap==null) statemap = new DefaultStateMap();
        //csc_022502.1_start
        if (session!=null) {
            StateMap sm = (StateMap) session.getAttribute(LOCAL_OR);
            if (sm!=null) {
                List list = sm.getStateKeys();
                if (list!=null) {
                    ObjectRepository lr = ObjectRepository.getLocalRepository();
                    Iterator it = list.iterator();
                    while (it.hasNext()) {
                        Object key = it.next();
                        Object val = sm.getState(key);
                        lr.putState(key, val);
                    }
                }
                session.removeAttribute(LOCAL_OR);
            }
        }
        //csc_022502.1_end
    }
    
    //-------------------- StateMap ------------------------------
    /**
     * set a property in this StateMap
     *
     * @param key the state key object
     * @param val the state value object
     */    
    public void putState(Object key, Object val) {
        statemap.putState(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the state key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        return statemap.getState(key);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        return statemap.removeState(key);
    }
    
    /**
     * get a list of the keys for this StateMap
     *
     * @return a list the keys for this StateMap
     */    
    public List getStateKeys() {
        return statemap.getStateKeys();
    }
    
    /**
     * get a copy of the underlying Map
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        return statemap.getStateValues();    
    }
    
}
