/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultEventGateway.java,v 1.8 2002/05/29 06:17:35 JacobK Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

import org.apache.log4j.*;

import org.enhydra.barracuda.core.util.data.Collections;
import org.apache.log4j.*;

/**
 * Default implementation of an EventGateway. An event
 * gateway just indicates that the class has an interest in
 * events, and as such needs access to the EventBroker in 
 * order to register its listeners there.
 */
public class DefaultEventGateway implements EventGateway {

    //public constants
    protected static Logger logger = Logger.getLogger(DefaultEventGateway.class.getName());
    
    private static byte[] sep = System.getProperty("line.separator").getBytes();

    /**
     * @supplierCardinality 0..* 
     */
    private EventGateway parent = null;
    private List gateways = new ArrayList();
    private List interests = null;
    private List aliases = null;

    /**
     * Set the parent event gateway. Null indicates its the root.
     *
     * @param eg the parent event gateway
     */
    public final void setParent(EventGateway eg) {
        parent = eg;
    }
    
    /**
     * Get the parent event gateway. Returns null if it's the root.
     *
     * @return the parent event gateway
     */
    public final EventGateway getParent() {
        return parent;    
    }
    
    /**
     * Add an event gateway to this one
     *
     * @param eg the event gateway to be added
     */
    public final void add(EventGateway eg) {
        if (eg==null) return;
        if (logger.isInfoEnabled()) logger.info(this+" adding "+eg);
        gateways.add(eg);
        eg.setParent(this);
    }
    
    /**
     * Remove an event gateway from this one
     *
     * @param eg the event gateway to be removed
     */
    public final void remove(EventGateway eg) {
        if (eg==null) return;
        if (logger.isInfoEnabled()) logger.info(this+" removing "+eg);
        gateways.remove(eg);
        eg.setParent(null);
    }
    
    /**
     * Get a list of child gateways. The list returned is a copy of the
     * underlying child gateway list.
     *
     * @return a list of child gateways
     */
    public List getChildren() {
        return new ArrayList(gateways);
    }

    /**
     * Ask all interested parties to register with
     * the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        register for events
     */
    public final void register(EventBroker eb) {
        if (logger.isInfoEnabled()) logger.info(this+" registering with EventBroker");
        
        //register any specified interests
        if (interests!=null) {
            Iterator it = interests.iterator();
            while (it.hasNext()) {
                Interest interest = (Interest) it.next();
                try {
                    if (interest.eventClass!=null) eb.addEventListener(interest.factory, interest.eventClass);
                    else eb.addEventListener(interest.factory);
                } catch (org.enhydra.barracuda.core.event.InvalidClassException e) {}
            }
        }
        
        //register any specified aliases
        if (aliases!=null) {
            Iterator it = aliases.iterator();
            while (it.hasNext()) {
                Class event = (Class) it.next();
                try {eb.addEventAlias(event);} 
                catch (org.enhydra.barracuda.core.event.InvalidClassException e) {}
            }
            aliases = null;
        }
        
        //register local interests
        if (logger.isDebugEnabled()) logger.debug(this+"...registering local interests ");
        registerLocalEventInterests(eb);

        //register local aliases        
        if (logger.isDebugEnabled()) logger.debug(this+"...registering local aliases");
        registerLocalEventAliases(eb);

        //register child gateways
        if (logger.isDebugEnabled()) logger.debug(this+"...registering children ");
        Iterator it = gateways.iterator();
        while (it.hasNext()) {
            EventGateway eg = (EventGateway) it.next();
            if (logger.isDebugEnabled()) logger.debug(this+"...found child gateway:"+eg);
            eg.register(eb);
        }

        if (logger.isInfoEnabled()) logger.info("Registration complete!");
    }

    /**
     * Register any local interests in the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        register for local events
     */
    public void registerLocalEventInterests(EventBroker eb) {
        //this is where users would extend and override
    }

    /**
     * Register any local event aliases in the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        register aliases for local events
     */
    public void registerLocalEventAliases(EventBroker eb) {
        //this is where users would extend and override
    }

    /**
     * Rather than overriding the registerLocalEventInterests 
     * method, you can just invoke this method instead for each 
     * interest you'd like to register. The gateway will keep track
     * of all factories specified, and register/deregister when 
     * appropriate (so you don't have to worry about it). Notice that
     * this method registers just the listener id (not for a specific
     * class of event).
     *
     * The only real reason for using the registerLocalEventInterests 
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which 
     * throws an exception).
     *
     * @param factory the factory we wish to register with the event broker
     * @param event the class of events we are interested in
     */
    public final void specifyLocalEventInterests(ListenerFactory factory) {
        specifyLocalEventInterests(factory, null);
    }
    
    /**
     * Rather than overriding the registerLocalEventInterests 
     * method, you can just invoke this method instead for each 
     * interest you'd like to register. The gateway will keep track
     * of all factories specified, and register/deregister when 
     * appropriate (so you don't have to worry about it). 
     *
     * The only real reason for using the registerLocalEventInterests 
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which 
     * throws an exception).
     *
     * @param factory the factory we wish to register with the event broker
     * @param event the class of events we are interested in
     */
    public final void specifyLocalEventInterests(ListenerFactory factory, Class event) {
        if (logger.isDebugEnabled()) logger.debug(this+" specifying event interest: "+factory+"-->"+(event!=null ? event.getName() : "null"));
        if (interests==null) interests = new ArrayList();
        interests.add(new Interest(factory, event));
    }

    /**
     * Rather than overriding the registerLocalEventAliases 
     * method, you can just invoke this method instead for type
     * of event you want to manually alias.
     *
     * The only real reason for using the registerLocalEventAliases 
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which 
     * throws an exception).
     *
     * @param event the class of events we are interested in registering 
     *        aliases for
     */
    public final void specifyLocalEventAliases(Class event) {
        if (logger.isDebugEnabled()) logger.debug(this+" specifying event alias: "+event.getName());
        if (aliases==null) aliases = new ArrayList();
        aliases.add(event);    
    }

    /**
     * Ask all interested parties to de-register with
     * the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        de-register for events
     */
    public final void deregister(EventBroker eb) {
        if (logger.isInfoEnabled()) logger.info(this+" deregistering with EventBroker");
        
        //deregister any specified interests
        if (interests!=null) {
            Iterator it = interests.iterator();
            while (it.hasNext()) {
                Interest interest = (Interest) it.next();
                try {
                    if (interest.eventClass!=null) eb.removeEventListener(interest.factory, interest.eventClass);
                    else eb.removeEventListener(interest.factory);
                } catch (org.enhydra.barracuda.core.event.InvalidClassException e) {}
            }
            interests = null;
        }
        
        //deregister local interests
        if (logger.isDebugEnabled()) logger.debug(this+"...deregistering local ");
        deregisterLocalEventInterests(eb);
        
        //deregister child gateways
        if (logger.isDebugEnabled()) logger.debug(this+"...deregistering children ");
        Iterator it = gateways.iterator();
        while (it.hasNext()) {
            EventGateway eg = (EventGateway) it.next();
            if (logger.isDebugEnabled()) logger.debug(this+"...found child gateway:"+eg);
            eg.deregister(eb);
        }

        if (logger.isInfoEnabled()) logger.info("Deregistration complete!");
    }
    
    /**
     * Deregister any local interests in the EventBroker
     *
     * @param eb the event broker this gateway should use to 
     *        de-register for local events
     */
    public void deregisterLocalEventInterests(EventBroker eb) {
        //this is where users would extend and override
    }
    
    class Interest {
        ListenerFactory factory = null;
        Class eventClass = null;
        
        public Interest(ListenerFactory ifactory, Class ieventClass) {
            factory = ifactory;
            eventClass = ieventClass;        
        }
        
        public String toString() {
            return "Interest {factory:"+factory+" event:"+eventClass+"}";
        }
    }

    /**
     * For debugging purposes. Print the basic structure of the
     * gateway. 
     */
    public void printStackTrace(int depth, Category category) {
        printStackTrace(depth, category, null);
    }
    
    /**
     * For debugging purposes. Print the basic structure of the
     * gateway. 
     */
    public void printStackTrace(int depth, OutputStream out) {
        printStackTrace(depth, null, out);
    }
    
    /**
     * For debugging purposes. Print the basic structure of the
     * gateway. Output is printed to output stream, or if that is null,
     * to the default logger.
     */
    private void printStackTrace(int depth, Category category, OutputStream out) {
        if (depth<0) depth = 0;
        if (depth>25) depth = 25;
        String spaces = "                                                                              ";
        String inset = spaces.substring(0,depth*3);
        
        print(category, out, inset+this.getClass().getName() + "@" + Integer.toHexString(this.hashCode()));            

        //interests
        print(category, out, inset+"   interests: "+(interests==null ? "null" : ""));
        if (interests!=null) Collections.printStackTrace(interests, depth+2, category, out);
        print(category, out, inset+"   /end interests");

        //aliases
        print(category, out, inset+"   aliases: "+(aliases==null ? "null" : ""));
        if (aliases!=null) Collections.printStackTrace(aliases, depth+2, category, out);
        print(category, out, inset+"   /end aliases");

        //sub-gateways
        print(category, out, inset+"   gateways: "+(gateways==null ? "null" : ""));
        int cntr = -1;
        if (gateways!=null) {
            Iterator it = gateways.iterator();
            while (it.hasNext()) {
                EventGateway eg = (EventGateway) it.next();
                if (eg instanceof DefaultEventGateway) {  
                    print(category, out, inset+"      ["+(++cntr)+"]:");
                    ((DefaultEventGateway) eg).printStackTrace(depth+2, category, out);            
                } else {
                    print(category, out, inset+"      ["+(++cntr)+"]"+eg.getClass().getName()+" (details unknown)");
                }
            }                
        }
        print(category, out, inset+"   /end gateways");

        print(category, out, inset+"/end @" + Integer.toHexString(this.hashCode()));
    }

    protected static void print(Category category, OutputStream out, String s) {
        if (category!=null) {
            logger.debug(s);
        } else if (out!=null) {
            try {
                out.write(s.getBytes());
                out.write(sep);
            } catch (IOException ioe) {}
        }
    }
}
