/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: FormElement.java,v 1.10 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.util.data.DefaultStateMap;
import org.enhydra.barracuda.core.util.data.StateMap;

/**
 * <p>This interfaces defines the methods a class needs to implement
 * to act as a FormElement.
 *
 * <p>A FormElement defines how an element in a FormMap should
 * be mapped to a first class java object. There are several
 * key pieces of information required:
 *
 * <ul>
 *         <li>key - used to retrieve the target value from the incoming
 *            data source (either a ServletRequest or a StateMap)</li>
 *         <li>type - the type of data object the incoming value should be
 *            mapped to, as defined by the FormType class (String, Boolean,
 *            Integer, Date, Long, Short, Double, Float)</li>
 *         <li>default val - the default val to be used if the key value is
 *            null in the incoming data source (may be null)</li>
 *         <li>validator - any validators associated with this particular
 *            form elmenent (may be null)</li>
 *         <li>allow multiples - the servlet interface allows multiple params
 *            with the same key. This value indicates whether or not that is
 *            acceptable for a key. If so, then when multiple values are
 *            encountered they element will actually contain a List of values
 *            of the proper type</li>
 * </ul>
 *
 * <p>Once we have this information, we basically have enough data to map
 * a String value in the data source to a first class Java object in the
 * element.
 */
public interface FormElement {

    /**
     * Set the key value for this form element
     *
     * @param key the key value for this form element
     */
    public void setKey(String key);

    /**
     * Get the key value for this form element
     *
     * @return the key for this form element
     */
    public String getKey();

    /**
     * Set the name of this form element
     *
     * @param name the name value of this form element
     */
    public void setName(String name);

    /**
     * Get the name value of form element
     *
     * @return the name of this form element
     */
    public String getName();

    /**
     * Set the FormType for this form element
     *
     * @param type the FormType for this form element
     */
    public void setType(FormType type);
    
    /**
     * Get the FormType for this for element
     *
     * @return the FormType for this form element
     */
    public FormType getType();

    /**
     * Set the default value for this form element
     *
     * @param defaultVal the FormType for this form element
     */
    public void setDefaultVal(Object defaultVal);
    
    /**
     * Get the default value for this form element
     *
     * @return the default value for this form element (may be null)
     */
    public Object getDefaultVal();

    /**
     * Set whether or not this element allows multiple values
     *
     * @param val true if the element allows multiples
     */
    public void setAllowMultiples(boolean val);

    /**
     * Does this element allow multiple values
     *
     * @return true if this element allows multiple values
     */
    public boolean allowMultiples();

    /**
     * Set the FormValidator for this form element
     *
     * @param validator the FormValidator for this form element
     */
    public void setValidator(FormValidator validator);

    /**
     * Get the default FormValidator for this form element
     *
     * @return the validator for this form element (may be null)
     */
    public FormValidator getValidator();

    /**
     * Set the original value for this element
     *
     * @param origVal the original value
     */
    public void setOrigVal(Object iorigVal);

    /**
     * Get the original value for this element
     *
     * @return the original value for this form element (may be null)
     */
    public Object getOrigVal();

    /**
     * Set the value for this element
     *
     * @param value the value for this element
     */
    public void setVal(Object ival);

    /**
     * Get the value for this element
     *
     * @return the value for this form element (may be null)
     */
    public Object getVal();
    
    /**
     * Set the parse exception associated with this element
     *
     * @return any parse exceptions associated with the element
     */
    public void setParseException(ParseException pe);

    /**
     * Get any parse exceptions associated with the element (ie. that
     * might have occurred when the element was mapped)
     *
     * @return any parse exceptions associated with the element
     */
    public ParseException getParseException();
    
    //--------------- Convenience methods ------------------------
    /**
     * Get a String value from the map
     */
//    public String getStringVal();

    /**
     * Get a String value from the map, defaulting accordingly if
     * the value is null
     */
//    public String getStringVal(String dflt);

    /**
     * Get an Boolean value from the map
     */
//    public Boolean getBooleanVal();

    /**
     * Get an Integer value from the map
     */
//    public Integer getIntegerVal();

    /**
     * Get an Date value from the map
     */
//    public Date getDateVal();

    /**
     * Get an Long value from the map
     */
//    public Long getLongVal();

    /**
     * Get an Short value from the map
     */
//    public Short getShortVal();

    /**
     * Get an Double value from the map
     */
//    public Double getDoubleVal();

    /**
     * Get an Float value from the map
     */
//    public Float getFloatVal();

    /**
     * @clientCardinality 1
     */
    /*#FormValidator lnkFormValidator;*/
}