/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Iman L Crawford
 *
 * $Id: ListValidator.java,v 1.3 2002/05/31 14:48:32 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator validates all items even if OrigVal is an ArrayList
 *
 * @author  Iman L Crawford (icrawford@greatnation.com)
*/
public class ListValidator extends AbstractFormValidator {

    protected FormValidator fv = null;

    /**
     * Public constructor.
     *
     * @param fv the validator we wish to make sure is not valid
     */
    public ListValidator(FormValidator ifv) {
        this(ifv, null);
    }

    /**
     * Public constructor.
     *
     * @param fv the validator we wish to make sure is not valid
     * @param errmsg the message associated with this error
     */
    public ListValidator(FormValidator ifv, String ierrmsg) {
        setErrorMessage(ierrmsg);
        fv = ifv;
    }

    /**
     * Return the value that is being not'ed
     *
     * @return The form element to negate
     */
    public FormValidator getSubValidator() {
       return fv;
    }

    /**
     * Will check origVal's type and loop through all the values calling the
     * subValidator.
     *
     * @param val the value to validate, could be an ArrayList
     * @param element the form element to be validated (null indicates
     *        we want to perform form level validation)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validate(FormElement element, FormMap formMap, boolean deferExceptions) throws ValidationException {
        if (localLogger.isDebugEnabled()) localLogger.debug("Validating one or more items");

        if (element.getOrigVal() instanceof ArrayList) {
          ArrayList origList = (ArrayList)element.getOrigVal();

          for (int i = 0; i < origList.size(); i++) {
            FormElement newElement = this.getNewElement(i, element);

            fv.validate((FormElement)newElement, formMap, deferExceptions);
          }
        }
        else {
          if (fv != null) {
            fv.validate(element, formMap, deferExceptions);
          }
        }
    }


    /**
     * Will return a duplicate of the element passed to it.
     *
     * @param index index of value to map from the original element.
     * @param element the original form element to be validated
     */
  private FormElement getNewElement(int index, FormElement element) {
    DefaultFormElement newElement = new DefaultFormElement();
    Object origVal = ((ArrayList)element.getOrigVal()).get(index);
    Object val     = ((ArrayList)element.getVal()).get(index);

    newElement.setOrigVal(origVal);
    newElement.setVal(val);
    newElement.setKey(element.getKey());
    newElement.setName(element.getName());
    newElement.setType(element.getType());
    newElement.setDefaultVal(element.getDefaultVal());
    newElement.setAllowMultiples(false);
    newElement.setValidator(fv);

    try {
        // don't need to parse if origVal is the expected type
        Class typeclass = newElement.getType().getFormClass();
        if (!typeclass.isInstance(origVal))
          element.getType().parse(origVal.toString());
    } catch (ParseException e) {
        if (localLogger.isDebugEnabled()) localLogger.debug("ParseException:", e);
        newElement.setParseException(e);
    }


    return (FormElement)newElement;

  }

}
