/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: RangeValidator.java,v 1.9 2002/05/31 14:48:32 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that a value is within a given number range
 */
public class RangeValidator extends DefaultFormValidator {

    protected Number n1 = null;
    protected Number n2 = null;

    /**
     * Public no-args constructor.
     */
    public RangeValidator() {
        this(null, null, null);
    }

    /**
     * Public constructor.
     *
     * @param lo the low end of the range
     * @param high the high end of the range
     */
    public RangeValidator(Number in1, Number in2) {
        this(in1, in2, null);
    }

    /**
     * Public constructor.
     *
     * @param lo the low end of the range
     * @param high the high end of the range
     * @param errmsg the message associated with this error
     */
    public RangeValidator(Number in1, Number in2, String ierrmsg) {
        super(ierrmsg);
        n1 = in1;
        n2 = in2;
    }

    //bw_102501.1 - added
    /**
     * Get the minimum allowed value
     *
     * @return Number containing the minimum value
     */
    public Number getMinimum() {
       return n1;
    }

    //bw_102501.1 - added
    /**
     * Get the maximum allowed value
     *
     * @return Number containing the maximum value
     */
    public Number getMaximum() {
       return n2;
    }

    /**
     * Validate a FormElement to make see if the element equals() a
     * given object
     *
     * @param element the form element to be validated
     * @param element the form element that contains the val
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        //eliminate the obvious
        if (n1==null || n2==null) {
            throw this.generateException(element, deferExceptions, "Invalid range:"+n1+" to "+n2);
        }
        // cannot validate a null value, use NotNullvalidator if you want to disallow nulls
        // ilc_022702.1_start
        // use isNull to test nullness
        //if (val==null) return;
        if (this.isNull(val, element))
          return;
        // ilc_022702.1_end

        //figure out what type of element we're dealing with
        if (localLogger.isInfoEnabled()) localLogger.info("validating val="+val+" is in range between "+n1+" and "+n2);
        FormType formType = element.getType();
        String defaultErrMsg = "Value: "+val+ " is not in the range between "+n1+" and "+n2;
        // ilc_022702.2_start

        // make sure parsing was successful
        if (element.getParseException() == null) {
          if (formType==FormType.INTEGER) {
              // val is always origVal need to use getVal
              //int i = ((Integer) val).intValue();
              int i = ((Integer)element.getVal()).intValue();
              if (i<n1.intValue() || i>n2.intValue()) throw this.generateException(element, deferExceptions, defaultErrMsg);

          } else if (formType==FormType.LONG) {
              // val is always origVal need to use getVal
              //long l = ((Long) val).longValue();
              long l = ((Long)element.getVal()).longValue();
              if (l<n1.longValue() || l>n2.longValue()) throw this.generateException(element, deferExceptions, defaultErrMsg);

          } else if (formType==FormType.SHORT) {
              // val is always origVal need to use getVal
              //short s = ((Short) val).shortValue();
              short s = ((Short)element.getVal()).shortValue();
              if (s<n1.shortValue() || s>n2.shortValue()) throw this.generateException(element, deferExceptions, defaultErrMsg);

          } else if (formType==FormType.DOUBLE) {
              // val is always origVal need to use getVal
              //double d = ((Double) val).doubleValue();
              double d = ((Double)element.getVal()).doubleValue();
              if (d<n1.doubleValue() || d>n2.doubleValue()) throw this.generateException(element, deferExceptions, defaultErrMsg);

          } else if (formType==FormType.FLOAT) {
              // val is always origVal need to use getVal
              //float f = ((Float) val).floatValue();
              float f = ((Float)element.getVal()).floatValue();
              if (f<n1.floatValue() || f>n2.floatValue()) throw this.generateException(element, deferExceptions, defaultErrMsg);

          } else {
              throw this.generateException(element, deferExceptions, "Invalid form type:"+formType);
          }
        }
        else {
          throw this.generateException(element, deferExceptions, "Parse error " + element.getParseException());
        }
        // ilc_022702.2_end;
    }

}
