/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Collections.java,v 1.7 2002/02/21 22:10:20 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.util.*;
import java.io.*;

import org.apache.log4j.*;

/**
 * Collections utilities
 */
public class Collections {

    private static byte[] sep = System.getProperty("line.separator").getBytes();

    /**
     * <p>utility method to recursively print the stack trace for a Map. 
     *
     * @param map the Map we wish to dump
     */
    public static void printStackTrace(Map map) {
        printStackTrace(map, System.out);
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a Map. 
     *
     * @param map the Map we wish to dump
     * @param logger the Category we wish to dump it to
     */
    public static void printStackTrace(Map map, Category logger) {
        printStackTrace(map, 0, logger, null);
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a Map.
     *
     * @param map the Map we wish to dump
     * @param out the output stream we wish to dump it to
     */
    public static void printStackTrace(Map map, OutputStream out) {
        if (out==null) out = System.out;
        printStackTrace(map, 0, null, out);
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a Map</p>
     *
     * <p>Map Specification:
     * <ol>
     *   <li>depth < 0 will be remapped to 0</li>
     *   <li>depth > 25 will be remapped to 25</li>
     * </ol>
     *
     * <p>Passing in a null value for the output stream will cause all the 
     * necessary output information to be generated, but nothing will
     * actually print anywhere. This is useful for bounds testing when
     * you want to make sure the routine is working but you don't really 
     * care about the output.
     *
     * @param map the Map we wish to dump
     * @param int inset depth at which to start printing
     * @param logger the Category we wish to dump it to (may be null)
     * @param out OutputStream to print to (only used if logger is null)
     */
    public static void printStackTrace(Map map, int depth, Category logger, OutputStream out) {
        if (depth<0) depth = 0;
        if (depth>25) depth = 25;
        String spaces = "                                                                              ";
        String inset = spaces.substring(0,depth*3);
        print (logger, out, inset+map.getClass().getName() + "@" + Integer.toHexString(map.hashCode()));            
        print (logger, out, inset+"   properties: ");
        Object keys[] = map.keySet().toArray();
        if (keys!=null) {
            int max = keys.length;
            for (int i=0; i<max; i++) {
                Object value = map.get(keys[i]);
                if (value instanceof Map) {
                    print (logger, out, inset+"   [k]:"+keys[i]+" [v]: ...(Map)");
                    printStackTrace((Map) value, depth+2, logger, out);
                } else if (value instanceof List) {
                    List l = (List) value;
                    print (logger, out, inset+"   [k]:"+keys[i]+" [v]: ...(List - "+l.size()+" items)");
                    if (l.size()>0) printStackTrace(l, depth+2, logger, out);
                } else {
                    print (logger, out, inset+"   [k]:"+keys[i]+" [v]:"+value);
                }
            }
        }
        print (logger, out, inset+"   /end properties");
        print (logger, out, inset+"/end @" + Integer.toHexString(map.hashCode()));
        if (out!=null) try {out.flush();} catch (IOException ioe) {}
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a List. 
     * 
     * @param map the List we wish to dump
     */
    public static void printStackTrace(List list) {
        printStackTrace(list, System.out);
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a List. 
     * 
     * @param map the List we wish to dump
     * @param logger the Category we wish to dump it to
     */
    public static void printStackTrace(List list, Category logger) {
        printStackTrace(list, 0, logger, null);
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a List.
     *
     * @param map the Map we wish to dump
     * @param out the output stream we wish to dump it to
     */
    public static void printStackTrace(List list, OutputStream out) {
        if (out==null) out = System.out;
        printStackTrace(list, 0, null, out);
    }
    
    /**
     * <p>utility method to recursively print the stack trace for a List</p>
     *
     * <p>Bounds: <br>
     * If depth < 0, the method returns immediately</p>
     *
     * @param map the List we wish to dump
     * @param int inset depth at which to start printing
     * @param logger the Category we wish to dump it to (may be null)
     * @param out OutputStream to print to (only used if logger is null)
     */
    public static void printStackTrace(List list, int depth, Category logger, OutputStream out) {
        if (depth<0) depth = 0;
        if (depth>25) depth = 25;
        String spaces = "                                                                              ";
        String inset = spaces.substring(0,depth*3);
        print (logger, out, inset+list.getClass().getName() + "@" + Integer.toHexString(list.hashCode()));            
        print (logger, out, inset+"   items: ");
        Object items[] = list.toArray();
        if (items!=null) {
            int max = items.length;
            for (int i=0; i<max; i++) {
                if (items[i] instanceof Map) {
                    print (logger, out, inset+"   ["+i+"]: ...(Map)");
                    printStackTrace((Map) items[i], depth+2, logger, out);
                } else if (items[i] instanceof List) {
                    List l = (List) items[i];
                    print (logger, out, inset+"   ["+i+"]: ...(List - "+l.size()+" items)");
                    if (l.size()>0) printStackTrace(l, depth+2, logger, out);
                } else {
                    print (logger, out, inset+"   ["+i+"]: "+items[i]);
                }
            }
        }
        print (logger, out, inset+"   /end items");
        print (logger, out, inset+"/end @" + Integer.toHexString(list.hashCode()));
        if (out!=null) try {out.flush();} catch (IOException ioe) {}
    }
    
    protected static void print(Category logger, OutputStream out, String s) {
        if (logger!=null) {
            logger.debug(s);
        } else if (out!=null) {
            try {
                out.write(s.getBytes());
                out.write(sep);
            } catch (IOException ioe) {}
        }
    }

}
