/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Jacob Kjome
 *
 * $Id: Log4jInit.java,v 1.3 2002/06/13 07:26:49 JacobK Exp $
 */
package org.enhydra.barracuda.core.util.logging;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.PrintWriter;
import java.io.IOException;
import java.io.File;
import java.io.InputStream;
import java.util.Properties;

import org.apache.log4j.helpers.LogLog;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.xml.DOMConfigurator;

import org.apache.xerces.parsers.DOMParser;

import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

/**
 * A servlet for initializing Log4j. See
 * <a href="http://jakarta.apache.org/log4j/docs/documentation.html">Log4j documentation</a>
 * for how to use Log4j.
 * <p>
 * This servlet is never called by a client, but should be called during
 * web application initialization, i.e. when the servlet engine starts. The
 * following code should be inserted in the web.xml file for the web
 * application:
 * <p>
 * <pre>
 * 	<servlet>
 * 		<servlet-name>log4j-init</servlet-name>
 * 	    <servlet-class>org.enhydra.barracuda.core.util.logging.Log4jInit</servlet-class>
 *      <init-param>
 *           <param-name>log4j-config</param-name>
 *           <param-value>WEB-INF/log4j.xml</param-value>
 *      </init-param>
 *      <init-param>
 *           <param-name>log4j-cron</param-name>
 *           <param-value>5000</param-value>
 *		</init-param>
 *      <init-param>
 *           <param-name>log4j-log-home</param-name>
 *           <param-value>/usr/local/logs/tomcat</param-value>
 *		</init-param>
 *      <load-on-startup>1</load-on-startup>
 *	</servlet>
 * </pre>
 * <p>
 * The <code>log4j-config</code> init parameter specifies the location of the
 * Log4j configuration file relative to the current webapp.
 * If the <code>log4j-config</code> init parameter is omitted, this class
 * will just let Log4j configure itself since, upon first use of Log4j, if it
 * has not yet been configured, it will search for a config file named log4j.xml
 * or log4j.properties in the classpath. If it can't find one, it falls back to using the
 * <code>BasicConfigurator.configure()</code> to initialize Log4j.
 * </p>
 * <p>
 * The <code>log4j-cron</code> init parameter specifies the number of milliseconds
 * to wait in between reads of the config file using <code>configureAndWatch()</code>.
 * If omitted, given a value of 0, or given a value that is other than something that
 * which can be converted to a Java long value a normal <code>configure()</code> is used.
 * </p>
 * <p>
 * The <code>log4j-log-home</code> init parameter is optional. It specifies a
 * custom path to a directory meant to contain log files for the current webapp
 * when using a <code>FileAppender</code>. If not specified, it will default to
 * using the location WEB-INF/logs to contain log files. If the directory doesn't
 * exist, it is created. A system parameter is then created in the following format:
 * <blockquote>
 *     <code>[webapp name].log.home</code>
 * </blockquote>
 * This can be referenced in an xml config file (not sure if it works for a properties
 * config file?) in the following fashion:
 * <blockquote>
 *     <code>&lt;param name=&quot;File&quot; value=&quot;${Barracuda.log.home}/main.log&quot; /&gt;</code>
 * </blockquote>
 * In this case, we are running in the &quot;Barracuda&quot; context and the &quot;main.log&quot; file
 * will get created in whatever directory path is specified by the system property
 * &quot;Barracuda.log.home&quot;.
 * </p>
 * <p>
 * <strong>Note</strong> that if the webapp is being run directly from a .war file, the automatic creation
 * of the WEB-INF/logs directory and [webapp name].log.home system property will *not* be
 * performed. In this case, you would have to provide a custom directory path for the
 * this to work. Also note that <code>configureAndWatch()</code> will not be used in the case
 * that the webapp is running directly from a .war file. <code>configure()</code> will be used
 * instead.
 * </p>
 *
 * @author Jacob Kjome <hoju@visi.com>
 */
public final class Log4jInit extends HttpServlet {

    private final static String PARAM_LOG4J_CONFIG_PATH = "log4j-config";
    private final static String PARAM_LOG4J_WATCH_INTERVAL = "log4j-cron";
    private final static String PARAM_LOG4J_LOG_HOME = "log4j-log-home";
    private final static String DEFAULT_LOG_HOME = "WEB-INF" + File.separator + "logs";

    private static Boolean CONFIGURED = Boolean.FALSE;

    public void init() throws ServletException {
        if (CONFIGURED.equals(Boolean.FALSE)) {
            String configPath = getInitParameter(PARAM_LOG4J_CONFIG_PATH);
            // if the log4j-config parameter is not set, then no point in trying
            if (configPath != null) {
                if (configPath.startsWith("/")) configPath = (configPath.length() > 1) ? configPath.substring(1) : "";
                // if the configPath is an empty string, then no point in trying
                if (configPath.length() >= 1) {
                    // set up log path System property
                    String logHome = getInitParameter(PARAM_LOG4J_LOG_HOME);
                    if (logHome != null) {
                        // set up custom log path system property
                        setFileAppenderSystemProperty(logHome, this);
                    }
                    boolean isXMLConfigFile = (configPath.endsWith(".xml")) ? true : false;
                    String contextPath = getServletContext().getRealPath("/");
                    if (contextPath != null) {
                        // The webapp is deployed directly off the filesystem,
                        // not from a .war file so we *can* do File IO.
                        // This means we can use configureAndWatch() to re-read
                        // the the config file at defined intervals.
                        // Now let's check if the given configPath actually exists.
                        if (logHome == null) {
                            // no log path specified in web.xml. Setting to default
                            logHome = contextPath+DEFAULT_LOG_HOME;
                            setFileAppenderSystemProperty(logHome, this);
                        }
                        String systemConfigPath = configPath.replace('/', File.separatorChar);
                        File log4jFile = new File(contextPath+systemConfigPath);
                        if (log4jFile.canRead()) {
                            log4jFile = null;
                            String timerInterval = getInitParameter(PARAM_LOG4J_WATCH_INTERVAL);
                            long timerIntervalVal = 0L;
                            if (timerInterval != null) {
                                try {
                                    timerIntervalVal = Integer.valueOf(timerInterval).longValue();
                                }
                                catch (NumberFormatException nfe) {}
                            }
                            synchronized (CONFIGURED) {
                                if (CONFIGURED.equals(Boolean.FALSE)) {
                                    log("Configuring Log4j from File: "+contextPath+systemConfigPath);
                                    if (timerIntervalVal > 0) {
                                        log("Configuring Log4j with watch interval: "+timerIntervalVal+"ms");
                                        if (isXMLConfigFile) {
                                            DOMConfigurator.configureAndWatch(contextPath+systemConfigPath, timerIntervalVal);
                                        }
                                        else {
                                            PropertyConfigurator.configureAndWatch(contextPath+systemConfigPath, timerIntervalVal);
                                        }
                                    }
                                    else {
                                        if (isXMLConfigFile) {
                                            DOMConfigurator.configure(contextPath+systemConfigPath);
                                        }
                                        else {
                                            PropertyConfigurator.configure(contextPath+systemConfigPath);
                                        }
                                    }
                                    CONFIGURED = Boolean.TRUE;
                                } //end CONFIGURED check
                            } //end syncronized block
                        }
                        else {
                            // The given configPath does not exist.  So, let's just let Log4j look for the
                            // default files (log4j.properties or log4j.xml) on its own.
                            displayConfigNotFoundMessage();
                        } //end log4jFile.canRead() check
                    }
                    else {
                        // The webapp is deployed from a .war file, not directly
                        // off the file system so we *cannot* do File IO.
                        // Note that we *won't* be able to use configureAndWatch() here
                        // because that requires an absolute system file path.
                        // Now let's check if the given configPath actually exists.
                        InputStream log4jInputStream = getServletContext().getResourceAsStream("/"+configPath);
                        if (log4jInputStream != null) {
                            synchronized (CONFIGURED) {
                                if (CONFIGURED.equals(Boolean.FALSE)) {
                                    log("Configuring Log4j from InputStream at path: /"+configPath);
                                    if (isXMLConfigFile) {
                                        DOMParser log4jDOMParser = new DOMParser();
                                        try {
                                            log4jDOMParser.parse(new InputSource(log4jInputStream));
                                            DOMConfigurator.configure(log4jDOMParser.getDocument().getDocumentElement());
                                            CONFIGURED = Boolean.TRUE;
                                        }
                                        catch (SAXException se) {}
                                        catch (IOException ioe) {}
                                    }
                                    else {
                                        Properties log4jProps = new Properties();
                                        try {
                                            log4jProps.load(log4jInputStream);
    				                        log4jInputStream.close();
    				                        PropertyConfigurator.configure(log4jProps);
    				                        CONFIGURED = Boolean.TRUE;
                                        }
                                        catch (IOException ioe) {}
                                    }
                                } //end CONFIGURED check
                            } //end syncronized block
                        }
                        else {
                            // The given configPath does not exist.  So, let's just let Log4j look for the
                            // default files (log4j.properties or log4j.xml) on its own.
                            displayConfigNotFoundMessage();
                        } //end log4jInputStream null check
                    } //end contextPath null check
                }
                else {
                    LogLog.error("Zero length Log4j config file path given.");
                    displayConfigNotFoundMessage();
                } //end configPath length check
            }
            else {
                LogLog.error("Missing log4j-config servlet parameter missing.");
                displayConfigNotFoundMessage();
            } //end configPath null check
        } //end CONFIGURED check
    } //end init() method

    private void displayConfigNotFoundMessage() {
        LogLog.warn("No Log4j configuration file found at given path. Falling back to Log4j auto-configuration.");
    }

    private void setFileAppenderSystemProperty(String logHome, ServletConfig config) {
        File logHomeDir = new File(logHome);
        if (logHomeDir.exists() || logHomeDir.mkdir()) {
            ServletContext context = config.getServletContext();
            String tempdir = ""+context.getAttribute("javax.servlet.context.tempdir");
            int lastSlash = tempdir.lastIndexOf(File.separator);
            if ((tempdir.length()-1) > lastSlash) {
                String logHomePropertyName = tempdir.substring(lastSlash+1) + ".log.home";
                context.log("Setting system property [ " + logHomePropertyName + " ] to [ " + logHome + " ]");
                System.setProperty(logHomePropertyName, logHome);
            }
        }
    }

    /**
	 * Throws a ServletException.
	 */
    public void doGet(HttpServletRequest req, HttpServletResponse res)
                                    throws ServletException, IOException {
        throw new ServletException("Servlet only used for Log4j initialization");
    }

    public void doPost(HttpServletRequest req, HttpServletResponse res)
                                    throws ServletException, IOException {
        doGet(req, res);
    }

    /**
	 * Returns the fully qualified class name concatenated with the CVS id.
	 */
    public String getServletInfo() {
        return this.getClass().getName()
               + " version: "
               + "$Id: Log4jInit.java,v 1.3 2002/06/13 07:26:49 JacobK Exp $";
    }

}
