/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ValidatorTestCase.java,v 1.8 2002/05/29 06:17:36 JacobK Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.testbed.*;
import org.enhydra.barracuda.core.util.data.StateMap;


/**
 * This test verifies that BLink works correctly.
 */
public abstract class ValidatorTestCase extends TestCase {

    /**
     * Public Constructor
     */
    public ValidatorTestCase(String name) {
        super(name);
    }
    
    //to fully test a Validator, we need to verify its behavior against
    //all supported types. Consequently, validator tests should implement
    //these methods
    public abstract void testString();
    public abstract void testBoolean();
    public abstract void testInteger();
    public abstract void testDate();
    public abstract void testLong();
    public abstract void testShort();
    public abstract void testDouble();
    public abstract void testFloat();

    /**
     * Validate a validator against valid cases (shouldn't generate any errors)
     */
    public void assertValid(String msg, DefaultFormValidator v, DefaultFormElement el, Object o) {
        try {
            v.validateFormElement(o, el, false);
            //good, there was no error
        } catch (ValidationException ve) {
            fail(msg + " - " + ve);
        }
    }

    /**
     * Validate a validator against invalid cases (we expect errors)
     */
    public void assertInvalid(String msg, DefaultFormValidator v, DefaultFormElement el, Object o) {
        try {
            v.validateFormElement(o, el, false);
            fail(msg);
        } catch (ValidationException ve) {
            //error is expected
        }
    }

    public void assertAllValid( DefaultFormValidator v, DefaultFormElement el, Object[] objs ) {
        for ( int i = 0; i < objs.length; i++ ) {
            assertValid( objs[i] + " should be valid", v, el, objs[i] );
        }

    }

    public void assertAllInvalid( DefaultFormValidator v, DefaultFormElement el, Object[] objs ) {
        for ( int i = 0; i < objs.length; i++ ) {
            assertInvalid( objs[i] + " should be invalid", v, el, objs[i] );
        }

    }


    // ilc_02202.1_start
    // send all the validation through a formmap.

    public void assertAllInvalid(FormValidator fv, StateMap sm, FormType ft){
      this.assertAll(fv, sm, ft, false);
    }

    public void assertAllValid(FormValidator fv, StateMap sm, FormType ft){
      this.assertAll(fv, sm, ft, true);
    }

    public void assertAll(FormValidator fv, StateMap sm, FormType ft, boolean assertValid) {
        DefaultFormMap fm = new DefaultFormMap();
        Iterator it = sm.getStateKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            fm.defineElement(new DefaultFormElement(key, ft, null, fv));
        }

        //mow map the StateMap into the FormMap
        fm.map(sm);

        it = sm.getStateKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            DefaultFormElement el = (DefaultFormElement) fm.getElement(key);
            DefaultFormValidator v = (DefaultFormValidator) el.getValidator();
            if (assertValid)
              assertValid("Error validating element for key:"+key+" value: "+el.getOrigVal(), v, el, el.getOrigVal());
            else
              assertInvalid("Error invalidating element for key:"+key+" value: "+el.getOrigVal(), v, el, el.getOrigVal());
        }

    }

    // ilc_02202.1_end
}
