/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestMinLengthValidator.java,v 1.7 2002/05/29 06:17:36 JacobK Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.data.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.testbed.*;


/**
 * This test verifies that MinLengthValidator works correctly.
 */
public class TestMinLengthValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestMinLengthValidator.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestMinLengthValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //create a form validator and excercise it
        MinLengthValidator v1 = new MinLengthValidator(-1);
        MinLengthValidator v2 = new MinLengthValidator(0);
        MinLengthValidator v3 = new MinLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

        // ilc_022602.1_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        // we've decided "" is equivalant of null
        sm.putState("valid string -1 length", null);
        sm.putState("valid string -1 length1", "");
        sm.putState("valid string -1 length1", "   ");
        assertAllValid(v1, sm, FormType.STRING);

        // Validate 0 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid string 0 length1", null);
        sm.putState("valid string 0 length2", "");
        assertAllValid(v2, sm, FormType.STRING);

        // Validate 5 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid string 5 length1", null);
        sm.putState("valid string 5 length2", "");
        sm.putState("valid string 5 length3", "12345");
        sm.putState("valid string 5 length4", "123456");
        assertAllValid(v3, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid string 5 length1", "1234");
        assertAllInvalid(v3, sm, FormType.STRING);
        // ilc_022602.1_end

        // ilc_022602.2_start
        /* use state maps instead
        //v1
        assertValid("Error validating null.length()>=-1", v1, el, null);
        assertValid("Error validating ''.length()>=-1", v1, el, "");
        assertValid("Error validating foo.length()>=-1", v1, el, "foo");
        assertValid("Error validating foofoo.length()>=-1", v1, el, "foofoo");
        
        //v2
        assertValid("Error validating null.length()>=0", v2, el, null);
        assertValid("Error validating ''.length()>=0", v2, el, "");
        assertValid("Error validating foo.length()>=0", v2, el, "foo");
        assertValid("Error validating foofoo.length()>=0", v2, el, "foofoo");
        
        //v3
        assertInvalid("Error invalidating null.length()>=5", v3, el, null);
        assertInvalid("Error invalidating ''.length()>=5", v3, el, "");
        assertInvalid("Error invalidating foo.length()>=5", v3, el, "foo");
        assertValid("Error validating foofoo.length()>=5", v3, el, "foofoo");
        */
        // ilc_022602.2_end
    }
    
    /**
     * Test Boolean types - this test ensures that if we pass in a
     * null FormElement or try to validate the length on a Boolean
     * it will always generate a ValidationException
     */
    public void testBoolean() {
        //create a form validator and excercise it
        MinLengthValidator v = new MinLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);
        Boolean bT = new Boolean(true);
        Boolean bF = new Boolean(false);

        // ilc_022602.3_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        sm.putState("valid bool 5 length", null);
        assertAllValid(v, sm, FormType.BOOLEAN);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid bool 5 length1", bT);
        sm.putState("invalid bool 5 length1", bF);
        assertAllInvalid(v, sm, FormType.BOOLEAN);

        // ilc_022602.3_end

        // ilc_022602.4_start
        // use StateMap to test
        /*
        //v
        assertInvalid("Error invalidating null element", v, null, "foo");
        assertInvalid("Error invalidating Boolean(true)", v, el, bT);
        assertInvalid("Error invalidating Boolean(false)", v, el, bF);
        */
        // ilc_022602.4_end
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //create a form validator and excercise it
        MinLengthValidator v1 = new MinLengthValidator(-1);
        MinLengthValidator v2 = new MinLengthValidator(0);
        MinLengthValidator v3 = new MinLengthValidator(4);
        DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
        Integer i = new Integer(1234);

        // ilc_022602.5_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        sm.putState("valid integer -1 length", null);
        assertAllValid(v1, sm, FormType.INTEGER);

        // Validate 0 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid integer 0 length", null);
        assertAllValid(v2, sm, FormType.INTEGER);

        // Validate 5 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid integer 5 length1", null);
        sm.putState("valid integer 5 length2", i);
        assertAllValid(v3, sm, FormType.INTEGER);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid integer 0 length", new Integer(123));
        assertAllInvalid(v3, sm, FormType.INTEGER);
        // ilc_022602.5_end

        // ilc_022602.6_start
        // use statemap instead
        /*
        //v1
        assertValid("Error validating null.length()>=-1", v1, el, null);
        assertValid("Error validating i.length()>=-1", v1, el, i);
        
        //v2
        assertValid("Error validating null.length()>=0", v2, el, null);
        assertValid("Error validating i.length()>=0", v2, el, i);
        
        //v3
        assertInvalid("Error invalidating null.length()>=5", v3, el, null);
        assertInvalid("Error invalidating i.length()>=5", v3, el, i);
        */
        // ilc_022602.6_end
    }

    /**
     * Test Date types - this test ensures that if we try to validate
     * the length on a Date it will always generate a ValidationException
     */
    public void testDate() {
        //create a form validator and excercise it
        MinLengthValidator v = new MinLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);
        Date d = new Date();

        // ilc_022602.7_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        sm.putState("valid date 5 length", null);
        assertAllValid(v, sm, FormType.DATE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid date 5 length1", d);
        assertAllInvalid(v, sm, FormType.BOOLEAN);

        // ilc_022602.7_end

        // ilc_022602.8_start
        // use StateMap to test
        /*
        //v
        assertInvalid("Error invalidating null date", v, el, null);
        assertInvalid("Error invalidating Date", v, el, d);
        */
        // ilc_022602.8_start
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //create a form validator and excercise it
        MinLengthValidator v1 = new MinLengthValidator(-1);
        MinLengthValidator v2 = new MinLengthValidator(0);
        MinLengthValidator v3 = new MinLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);
        Long l = new Long(123);

        // ilc_022602.9_start
        // use a StateMap for testing to make sure elements go through mapping
        Long l4 = new Long(1234);
        Long l5 = new Long(12345);
        Long l6 = new Long(123456);

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        sm.putState("valid long -1 length1", null);
        sm.putState("valid long -1 length2", l);
        sm.putState("valid long -1 length3", l4);
        sm.putState("valid long -1 length4", l5);
        sm.putState("valid long -1 length4", l6);
        assertAllValid(v1, sm, FormType.LONG);


        // validate 0 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long 0 length1", null);
        sm.putState("invalid long 0 length2", l);
        sm.putState("invalid long 0 length3", l4);
        sm.putState("invalid long 0 length4", l5);
        sm.putState("invalid long 0 length5", l6);
        assertAllValid(v2, sm, FormType.LONG);

        // validate 5 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long 5 length1", null);
        sm.putState("invalid long 5 length2", l5);
        sm.putState("invalid long 5 length3", l6);
        assertAllValid(v3, sm, FormType.LONG);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid long 5 length1", l4);
        assertAllInvalid(v3, sm, FormType.LONG);


        // ilc_022602.9_end

        // ilc_022602.10_start
        // use StateMap to test
        /*
        //v1
        assertValid("Error validating null.length()>=-1", v1, el, null);
        assertValid("Error validating l.length()>=-1", v1, el, l);
        
        //v2
        assertValid("Error validating null.length()>=0", v2, el, null);
        assertValid("Error validating l.length()>=0", v2, el, l);
        
        //v3
        assertInvalid("Error invalidating null.length()>=5", v3, el, null);
        assertInvalid("Error invalidating l.length()>=5", v3, el, l);
        */
        // ilc_022602.10_end
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //create a form validator and excercise it
        MinLengthValidator v1 = new MinLengthValidator(-1);
        MinLengthValidator v2 = new MinLengthValidator(0);
        MinLengthValidator v3 = new MinLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);
        Short s = new Short((short)123);

        // ilc_022602.11_start
        // use a StateMap for testing to make sure elements go through mapping

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        sm.putState("valid short -1 length", null);
        sm.putState("valid short -1 length2", s);
        assertAllValid(v1, sm, FormType.SHORT);

        // validate 0 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid short 0 length1", null);
        sm.putState("valid short 0 length2", s);
        assertAllValid(v2, sm, FormType.SHORT);

        // validate 5 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid short 5 length1", null);
        sm.putState("valid short 5 length2", s);
        assertAllValid(v1, sm, FormType.SHORT);

        // ilc_022602.11_end

        // ilc_022602.12_start
        // use StateMap to test
        //v1
        /*
        //v1
        assertValid("Error null.length()>=-1", v1, el, null);
        assertValid("Error validating s.length()>=-1", v1, el, s);
        
        //v2
        assertValid("Error validating null.length()>=0", v2, el, null);
        assertValid("Error validating s.length()>=0", v2, el, s);
        
        //v3
        assertInvalid("Error invalidating null.length()>=5", v3, el, null);
        assertInvalid("Error invalidating s.length()>=5", v3, el, s);
        */
        // ilc_022602.12_end
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //create a form validator and excercise it
        MinLengthValidator v1 = new MinLengthValidator(-1);
        MinLengthValidator v2 = new MinLengthValidator(0);
        MinLengthValidator v3 = new MinLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
        Double d = new Double(123);

        // ilc_022602.13_start
        // use a StateMap for testing to make sure elements go through mapping

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        sm.putState("valid double -1 length", null);
        sm.putState("valid double -1 length2", d);
        assertAllValid(v1, sm, FormType.DOUBLE);

        // validate 0 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid double 0 length1", null);
        sm.putState("valid double 0 length2", d);
        assertAllValid(v2, sm, FormType.DOUBLE);

        // validate 5 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid double 5 length1", null);
        assertAllValid(v1, sm, FormType.DOUBLE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid double 5 length2", d);
        assertAllValid(v3, sm, FormType.DOUBLE);


        // ilc_022602.13_end

        // ilc_022602.14_start
        // use StateMap to test
        /*
        //v1
        assertValid("Error null.length()>=-1", v1, el, null);
        assertValid("Error validating d.length()>=-1", v1, el, d);
        
        //v2
        assertValid("Error validating null.length()>=0", v2, el, null);
        assertValid("Error validating d.length()>=0", v2, el, d);
        
        //v3
        assertInvalid("Error invalidating null.length()>=5", v3, el, null);
        assertValid("Error validating d.length()>=5", v3, el, d);    //the reason this is valid is because the d.toString results in '123.0'
        */
        // ilc_022602.14_end
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //create a form validator and excercise it
        MinLengthValidator v1 = new MinLengthValidator(-1);
        MinLengthValidator v2 = new MinLengthValidator(0);
        MinLengthValidator v3 = new MinLengthValidator(5);
        DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);
        Float f = new Float(1.23);
        Float f2 = new Float(1.233);

        // ilc_022602.15_start
        // use a StateMap for testing to make sure elements go through mapping

        DefaultStateMap sm = new DefaultStateMap();

        // Validate -1 min length
        sm.putState("valid float -1 length", null);
        sm.putState("valid float -1 length2", f);
        sm.putState("valid float -1 length3", f2);
        assertAllValid(v1, sm, FormType.FLOAT);

        // validate 0 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid float 0 length1", null);
        sm.putState("valid float 0 length2", f);
        sm.putState("valid float 0 length3", f2);
        assertAllValid(v2, sm, FormType.FLOAT);

        // validate 5 min length
        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid float 5 length1", null);
        sm.putState("valid float 5 length2", f2);
        assertAllValid(v3, sm, FormType.FLOAT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("invalid float 5 length1", f);
        assertAllInvalid(v3, sm, FormType.FLOAT);


        // ilc_022602.15_end

        // ilc_022602.16_start
        // use StateMap to test
        /*
        //v1
        assertValid("Error validating null.length()>=-1", v1, el, null);
        assertValid("Error validating f.length()>=-1", v1, el, f);
        
        //v2
        assertValid("Error validating null.length()>=0", v2, el, null);
        assertValid("Error validating f.length()>=0", v2, el, f);
        
        //v3
        assertInvalid("Error invalidating null.length()>=5", v3, el, null);
        assertInvalid("Error invalidating f.length()>=5", v3, el, f);
        assertValid("Error validating f2.length()>=5", v3, el, f2);
        */
        // ilc_022602.16_start
    }
    
}
